@@ -16,80 +16,94 @@ class AttrDict(dict):
   __getattr__ = dict.__getitem__
 
 
-class Terrain:
+class staticproperty:
 
-  def __init__(self, materials, area):
-    self._map = np.zeros(area, np.uint8)
-    self._names = {i + 1: n for i, n in enumerate(materials)}
-    self._ids = {n: i + 1 for i, n in enumerate(materials)}
+  def __init__(self, function):
+    self.function = function
+
+  def __get__(self, instance, owner=None):
+    return self.function()
+
+
+class World:
+
+  def __init__(self, area, seed=None):
+    self._terrain = np.zeros(area, np.uint8)
+    self._material_names = {0: None}
+    self._material_ids = {None: 0}
+    self._objects = [None]
+    self._coords = np.zeros(area, np.uint32)
+    self._random = np.random.RandomState(seed)
 
   @property
   def area(self):
-    return self._map.shape
+    return self._terrain.shape
 
-  def reset(self):
-    self._map[:] = 0
+  @property
+  def random(self):
+    return self._random
 
-  def count(self, material):
-    return (self._map == self._ids[material]).sum()
+  @property
+  def objects(self):
+    yield from (obj for obj in self._objects if obj)
 
-  def nearby(self, pos, distance):
-    ids = set(self._map[
-        pos[0] - distance: pos[0] + distance,
-        pos[1] - distance: pos[1] + distance].flatten().tolist())
-    return tuple(self._names[x] for x in ids)
+  def reset(self, seed=None):
+    # TODO: Not really needed. Can just create new instance.
+    self._random = np.random.RandomState(seed)
+    self._terrain[:] = 0
+    self._objects = [None]
+    self._coords[:] = 0
 
   def __setitem__(self, pos, material):
-    self._map[pos] = self._ids[material]
+    if material not in self._material_ids:
+      id_ = len(self._material_ids)
+      self._material_ids[material] = id_
+      self._material_names[id_] = material
+    self._terrain[pos] = self._material_ids[material]
 
   def __getitem__(self, pos):
-    if not _inside((0, 0), pos, self.area):
-      return None
-    return self._names[self._map[tuple(pos)]]
+    if _inside((0, 0), pos, self.area):
+      material = self._material_names[self._terrain[tuple(pos)]]
+    else:
+      material = None
+    if not (0 <= pos[0] < self._coords.shape[0]):
+      obj = False
+    elif not (0 <= pos[1] < self._coords.shape[1]):
+      obj = False
+    else:
+      obj = self._objects[self._coords[tuple(pos)]]
+    return material, obj
 
+  def nearby(self, pos, distance):
+    # TODO: Return both nearby materials and objects.
+    ids = set(self._terrain[
+        pos[0] - distance: pos[0] + distance,
+        pos[1] - distance: pos[1] + distance].flatten().tolist())
+    return tuple(self._material_names[x] for x in ids)
 
-class Objects:
-
-  def __init__(self, area):
-    self._objects = [None]
-    self._map = np.zeros(area, np.uint32)
-
-  def reset(self):
-    self._objects = [None]
-    self._map[:] = 0
-
-  def __iter__(self):
-    yield from (obj for obj in self._objects if obj)
+  def count(self, material):
+    if material not in self._material_ids:
+      return 0
+    return (self._terrain == self._material_ids[material]).sum()
 
   def add(self, obj):
     assert hasattr(obj, 'pos')
-    assert self.free(obj.pos)
-    self._map[obj.pos] = len(self._objects)
+    obj.pos = np.array(obj.pos)
+    assert self[obj.pos][1] is None
+    self._coords[tuple(obj.pos)] = len(self._objects)
     self._objects.append(obj)
 
   def remove(self, obj):
-    self._objects[self._map[obj.pos]] = None
-    self._map[obj.pos] = 0
+    self._objects[self._coords[tuple(obj.pos)]] = None
+    self._coords[tuple(obj.pos)] = 0
 
   def move(self, obj, pos):
-    assert self.free(pos)
-    self._map[pos] = self._map[obj.pos]
-    self._map[obj.pos] = 0
+    pos = np.array(pos)
+    assert self[pos][1] is None
+    self._coords[tuple(pos)] = self._coords[tuple(obj.pos)]
+    self._coords[tuple(obj.pos)] = 0
     obj.pos = pos
 
-  def free(self, pos):
-    return self.at(pos) is None
-
-  def nearby(self, pos, types=None):
-    raise NotImplementedError
-
-  def at(self, pos):
-    if not (0 <= pos[0] < self._map.shape[0]):
-      return False
-    if not (0 <= pos[1] < self._map.shape[1]):
-      return False
-    return self._objects[self._map[pos]]
-
 
 class Textures:
 
@@ -126,14 +140,13 @@ class UncoverView:
 
 class LocalView:
 
-  def __init__(self, terrain, objects, textures, unit, grid):
-    self._terrain = terrain
-    self._objects = objects
+  def __init__(self, world, textures, unit, grid):
+    self._world = world
     self._textures = textures
     self._unit = np.array(unit)
     self._grid = np.array(grid)
     self._offset = self._grid // 2
-    self._area = np.array(self._terrain.area)
+    self._area = np.array(self._world.area)
     self._center = None
 
   def __call__(self, player):
@@ -144,9 +157,9 @@ class LocalView:
         pos = self._center + np.array([x, y]) - self._offset
         if not _inside((0, 0), pos, self._area):
           continue
-        texture = self._textures.get(self._terrain[pos], self._unit)
+        texture = self._textures.get(self._world[pos][0], self._unit)
         _draw(canvas, np.array([x, y]) * self._unit, texture)
-    for obj in self._objects:
+    for obj in self._world.objects:
       pos = obj.pos - self._center + self._offset
       if not _inside((0, 0), pos, self._grid):
         continue
