@@ -11,8 +11,8 @@ def main():
   parser = argparse.ArgumentParser()
   parser.add_argument('--seed', type=int, default=None)
   parser.add_argument('--area', nargs=2, type=int, default=(64, 64))
-  parser.add_argument('--view', type=int, default=4)
-  parser.add_argument('--size', type=int, default=64)
+  parser.add_argument('--view', type=int, nargs=2, default=(9, 9))
+  parser.add_argument('--size', type=int, nargs=2, default=(64, 64))
   parser.add_argument('--length', type=int, default=10000)
   parser.add_argument('--health', type=int, default=5)
   parser.add_argument('--record', type=str, default=None)
