@@ -16,12 +16,11 @@ def main():
   parser.add_argument('--filename', type=str, default='terrain.png')
   args = parser.parse_args()
 
-  view = max(args.area) // 2 - 1
-  env = crafter.Env(args.area, view, args.size, seed=args.seed)
+  env = crafter.Env(args.area, args.area, args.size, seed=args.seed)
   images = []
   for index in range(args.amount):
     images.append(env.reset()['image'])
-    diamonds = env._terrain.count('diamond')
+    diamonds = env._world.count('diamond')
     print(f'Map: {index:>2}, diamonds: {diamonds:>2}')
 
   rows = len(images) // args.cols
